@@ -63,11 +63,11 @@ var Presence = class {
     }
 
     setDefault() {
+        this._settings.set_boolean('show-banners', true);
+
         if (this._patch.applied) {
             this._patch.revert();
         }
-
-        this._settings.set_boolean('show-banners', true);
     }
 
     _onPatchApplied() {
@@ -82,7 +82,8 @@ var Presence = class {
 
     _onPatchReverted() {
         try {
-            let status = Main.messageTray._presence.status;
+            const status = Main.messageTray._presence.status;
+
             Main.messageTray._onStatusChanged(status);
         }
         catch (error) {
