@@ -18,11 +18,10 @@
  *
  */
 
-const Signals = imports.signals;
-
 const { Gio, Meta, Shell } = imports.gi;
 
 const Main = imports.ui.main;
+const Signals = imports.misc.signals;
 
 const Extension = imports.misc.extensionUtils.getCurrentExtension();
 
@@ -37,8 +36,10 @@ const VIDEO_PLAYER_CATEGORIES = [
 ];
 
 
-var Patch = class {
+var Patch = class extends Signals.EventEmitter {
     constructor(object, overrides) {
+        super();
+
         this.object = object;
         this.overrides = overrides;
         this.initial = {};
@@ -82,7 +83,6 @@ var Patch = class {
         this.disconnectAll();
     }
 };
-Signals.addSignalMethods(Patch.prototype);
 
 
 var TransitionGroup = class {
@@ -115,12 +115,14 @@ var TransitionGroup = class {
             return;
         }
 
-        this._actors.push({
+        const meta = {
             actor: actor,
             destroyId: actor.connect('destroy', () => {
                 this.removeActor(actor);
-            })
-        });
+                meta.destroyId = 0;
+            }),
+        };
+        this._actors.push(meta);
 
         if (!this._referenceActor) {
             this._setReferenceActor(actor);
@@ -130,9 +132,10 @@ var TransitionGroup = class {
     removeActor(actor) {
         let index = this._findActor(actor);
         if (index >= 0) {
-            let meta = this._actors.splice(index, 1);
-
-            actor.disconnect(meta.destroyId);
+            const meta = this._actors.splice(index, 1);
+            if (meta.destroyId) {
+                actor.disconnect(meta.destroyId);
+            }
         }
 
         if (this._referenceActor === actor) {
